/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v2

import (
	"context"
	"time"

	v2 "github.com/kyverno/kyverno/api/kyverno/v2"
	scheme "github.com/kyverno/kyverno/pkg/client/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// UpdateRequestsGetter has a method to return a UpdateRequestInterface.
// A group's client should implement this interface.
type UpdateRequestsGetter interface {
	UpdateRequests(namespace string) UpdateRequestInterface
}

// UpdateRequestInterface has methods to work with UpdateRequest resources.
type UpdateRequestInterface interface {
	Create(ctx context.Context, updateRequest *v2.UpdateRequest, opts v1.CreateOptions) (*v2.UpdateRequest, error)
	Update(ctx context.Context, updateRequest *v2.UpdateRequest, opts v1.UpdateOptions) (*v2.UpdateRequest, error)
	UpdateStatus(ctx context.Context, updateRequest *v2.UpdateRequest, opts v1.UpdateOptions) (*v2.UpdateRequest, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v2.UpdateRequest, error)
	List(ctx context.Context, opts v1.ListOptions) (*v2.UpdateRequestList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v2.UpdateRequest, err error)
	UpdateRequestExpansion
}

// updateRequests implements UpdateRequestInterface
type updateRequests struct {
	client rest.Interface
	ns     string
}

// newUpdateRequests returns a UpdateRequests
func newUpdateRequests(c *KyvernoV2Client, namespace string) *updateRequests {
	return &updateRequests{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the updateRequest, and returns the corresponding updateRequest object, and an error if there is any.
func (c *updateRequests) Get(ctx context.Context, name string, options v1.GetOptions) (result *v2.UpdateRequest, err error) {
	result = &v2.UpdateRequest{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("updaterequests").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of UpdateRequests that match those selectors.
func (c *updateRequests) List(ctx context.Context, opts v1.ListOptions) (result *v2.UpdateRequestList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v2.UpdateRequestList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("updaterequests").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested updateRequests.
func (c *updateRequests) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("updaterequests").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a updateRequest and creates it.  Returns the server's representation of the updateRequest, and an error, if there is any.
func (c *updateRequests) Create(ctx context.Context, updateRequest *v2.UpdateRequest, opts v1.CreateOptions) (result *v2.UpdateRequest, err error) {
	result = &v2.UpdateRequest{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("updaterequests").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(updateRequest).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a updateRequest and updates it. Returns the server's representation of the updateRequest, and an error, if there is any.
func (c *updateRequests) Update(ctx context.Context, updateRequest *v2.UpdateRequest, opts v1.UpdateOptions) (result *v2.UpdateRequest, err error) {
	result = &v2.UpdateRequest{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("updaterequests").
		Name(updateRequest.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(updateRequest).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *updateRequests) UpdateStatus(ctx context.Context, updateRequest *v2.UpdateRequest, opts v1.UpdateOptions) (result *v2.UpdateRequest, err error) {
	result = &v2.UpdateRequest{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("updaterequests").
		Name(updateRequest.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(updateRequest).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the updateRequest and deletes it. Returns an error if one occurs.
func (c *updateRequests) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("updaterequests").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *updateRequests) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("updaterequests").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched updateRequest.
func (c *updateRequests) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v2.UpdateRequest, err error) {
	result = &v2.UpdateRequest{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("updaterequests").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
